/* $Id: mbx_imp.h,v 1.5 2000/03/19 16:28:58 hurtta Exp $ */

/******************************************************************************
 *  The Elm (ME+) Mail System  -  $Revision: 1.5 $   $State: Exp $
 *
 *  Author: Kari Hurtta <hurtta+elm@ozone.FMI.FI>
 *****************************************************************************/

struct read_folder_state {
    long    fbytes;
    int     linecounter;
    long    fbytes_body;
    enum prepare_mode mode;

    union {
	struct FILE_rs {
	    char    * next_line;
	    int     next_line_len;
	} file;
#ifdef POP
	struct POP_MBX_rs {
	    int msg_num;              /* Current msg_num */
	    int data_idx;
	    char **uidl_vector;
	    int uidl_count;
	} pop_mbx;
#endif
    } a;
};

typedef void mbx_close_folder P_((struct folder_info *folder,
				  enum close_mode mode));
typedef int  mbx_lock_folder  P_((int direction,struct folder_info *folder));
typedef int  mbx_unlock_folder P_((int interrupt, struct folder_info *folder));
typedef void mbx_flush_folder P_((struct folder_info *folder));
typedef void mbx_init_folder P_((struct folder_info *folder));
typedef void mbx_free_folder P_((struct folder_info *folder));
typedef int mbx_sessionlock_folder P_((struct folder_info *folder,
				       enum sessionlock_mode mode));
typedef int mbx_ferror_folder P_((struct folder_info *folder, int clean));
typedef int mbx_prepare_read_folder P_((struct folder_info *folder,
					enum prepare_mode mode,
					READ_STATE read_state_ptr));
typedef int mbx_end_read_folder P_((struct folder_info *folder,
				    READ_STATE read_state_ptr,
				    int silent));
typedef int mbx_copy_envelope_folder P_((struct folder_info *folder,
					 READ_STATE read_state_ptr,
					 struct header_rec *entry));
typedef CONST char * mbx_is_forwarded_folder P_((struct folder_info *folder,
						 READ_STATE read_state_ptr));
typedef int mbx_copy_header_folder P_((struct folder_info *folder,
				       READ_STATE read_state_ptr,
				       char **buffer, int *len));
typedef int mbx_copy_body_folder P_((struct folder_info *folder,
				     READ_STATE read_state_ptr,
				     char **buffer, int *len,
				     long *content_remaining));
typedef int mbx_copy_envelope_end_folder P_((struct folder_info *folder,
					     READ_STATE read_state_ptr));
typedef int mbx_copy_envelope_reset_body P_((struct folder_info *folder,
					     READ_STATE read_state_ptr));
typedef FILE * mbx_folder_to_fd P_((struct folder_info *folder,long offset));
typedef int mbx_new_mail_on_folder P_((struct folder_info *folder,int *bytes));
typedef int mbx_consider_remove_folder P_((struct folder_info *folder));
typedef int mbx_prepare_keep_folder P_((struct folder_info *folder,
					KEEP_STATE keep_state_ptr));
typedef int mbx_end_keep_folder P_((struct folder_info *folder,
				    KEEP_STATE keep_state_ptr));
typedef void mbx_mark_keep_folder P_((struct folder_info *folder,
				      KEEP_STATE keep_state_ptr,
				      struct header_rec *entry,
				      int keep));
typedef CONST char * mbx_folder_type P_((struct folder_info *folder));
typedef int mbx_start_edit_folder P_((struct folder_info *folder, 
				      CONST char **buffer));
typedef int mbx_end_edit_folder P_((struct folder_info *folder));
typedef void mbx_zero_rs_fields_folder P_((struct read_folder_state *rs));
typedef void mbx_free_rs_fields_folder P_((struct read_folder_state *rs));
typedef void mbx_zero_ks_fields_folder P_((struct keep_folder_state *rs));
typedef void mbx_free_ks_fields_folder P_((struct keep_folder_state *rs));


struct folder_type {
    mbx_close_folder             * close_it;
    mbx_lock_folder              * lock_it;
    mbx_init_folder              * init_it;
    mbx_sessionlock_folder       * sessionlock_it;
    mbx_unlock_folder            * unlock_it;
    mbx_flush_folder             * flush_it;
    mbx_ferror_folder            * ferror_it;
    mbx_prepare_read_folder      * prepare_read_it;
    mbx_end_read_folder          * end_read_it;
    mbx_copy_envelope_folder     * copy_envelope_it;
    mbx_is_forwarded_folder      * is_forwarded_it;
    mbx_copy_header_folder       * copy_header_it;
    mbx_copy_body_folder         * copy_body_it;
    mbx_copy_envelope_end_folder * copy_envelope_end_it;
    mbx_copy_envelope_reset_body * copy_envelope_reset_it;
    mbx_folder_to_fd             * xxx_to_fd;
    mbx_new_mail_on_folder       * new_mail_on_it;
    mbx_consider_remove_folder   * consider_remove_it;
    mbx_prepare_keep_folder      * prepare_keep_it;
    mbx_end_keep_folder          * end_keep_it;
    mbx_mark_keep_folder         * mark_keep_it;
    mbx_folder_type              * type;
    mbx_start_edit_folder        * start_edit_it;
    mbx_end_edit_folder          * end_edit_it;
    mbx_free_folder              * free_it;
    mbx_zero_rs_fields_folder    * zero_rs_fields_it;
    mbx_free_rs_fields_folder    * free_rs_fields_it;
    mbx_zero_ks_fields_folder    * zero_ks_fields_it;  
    mbx_free_ks_fields_folder    * free_ks_fields_it;
};

extern int mbx_copy_line_to_temp P_((struct folder_info *folder,
				     char *buffer, int len));
extern void append_buffer P_((char **buffer, int *len,
			      char *buffer1, int len1));
extern void mbx_flush_temp P_((struct folder_info *folder));

struct keep_folder_state {
    union {
	struct FILE_ks {
	    char * temp_keep_file;
	    FILE * keep_file;
	} file;
    } a;
};

#ifdef POP
extern struct folder_type pop_mbx;

#ifdef I_NETINET_IN
#include <netinet/in.h>
#endif

typedef union socket_addr
{
    struct sockaddr         sa;     /* general version */
#ifdef I_NETINET_IN
    struct sockaddr_in      sin;    /* INET family */
#endif
} SOCKADDR;

typedef enum { POP_error = 0,
	       POP_idle, POP_simple_command, POP_simple_response,
	       POP_multiline_command, POP_multiline_response, 
	       POP_multiline_data,
	       POP_command_ready } pop_states;

#endif

struct private_data {
    FILE *fh_temp;		   /* current folder 	     */
    FILE *fh_folder;               /* current folder 	     */

    union {
	struct SPOOL {
	    int  lock_state;
#ifdef	USE_DOTLOCK_LOCKING
	    char *lockfile;
#endif  /* USE_DOTLOCK_LOCKING */
	} spool;
#ifdef POP
	struct POP_MBX {
	    SOCKADDR  hostaddr;

	    int        socket;
	    pop_states pop_state;

	    char      *read_buffer;
	    int       read_len;

	    char      *write_buffer;
	    int       write_len;

	    /* Last command */
	    char      command[4];

	    /* parsed response */
	    char      *command_status;
	    char      *command_data;
	    int       data_len;

	    /* size from STAT */
	    int       stat_size;
	    int       stat_count;

	    struct uidl_entry  * uidl_root; /* Binary tree -- not balanced! */
	} pop_mbx;
#endif /* POP */
    } a;
};

struct mbx_hdr_info {
    struct info_type * type_code;
    union {
	int dummy;
#ifdef POP
	struct POP_MBX_if {
	    int msg_num;
	    char * uidl;   
	} pop_mbx;
#endif
    } a;
};

typedef void info_zero_routine P_((struct mbx_hdr_info *info));
typedef void info_free_routine P_((struct mbx_hdr_info *info));

typedef struct info_type {
    info_zero_routine    * zero_it;
    info_free_routine    * free_it;
}  * info_type_t;

void change_rec_mbx_info P_((struct header_rec *entry,
			     info_type_t t));


/*
 * Local Variables:
 *  mode:c
 *  c-basic-offset:4
 * End:
 */




