static char rcsid[] = "@(#)$Id: safemalloc.c,v 1.5 2000/02/11 18:21:37 hurtta Exp $";

/******************************************************************************
 *  The Elm (ME+) Mail System  -  $Revision: 1.5 $   $State: Exp $
 *
 *  Modified by: Kari Hurtta <hurtta+elm@ozone.FMI.FI>
 ******************************************************************************
 *  The Elm Mail System 
 *
 * 			Copyright (c) 1992 USENET Community Trust
 *****************************************************************************/

#include "headers.h"

/*
 * These routines perform dynamic memory allocation with error checking.
 * The "safe_malloc_fail_handler" vector points to a routine that is invoked
 * if memory allocation fails.  The default error handler displays a message
 * and aborts the program.
 */


void dflt_safe_malloc_fail_handler(proc, len)
     char *proc;
     unsigned len;
{
	fprintf(stderr,
		"error - out of memory [%s failed allocating %d bytes]\n",
		proc, len);
	exit(1);
}

void (*safe_malloc_fail_handler)() = dflt_safe_malloc_fail_handler;

malloc_t safe_malloc(len)
     unsigned len;
{
	malloc_t p;
	if ((p = malloc(len)) == NULL)
		(*safe_malloc_fail_handler)("safe_malloc", len);
	dprint(100,(debugfile,"safe_malloc(%d)=%X\n",len,p));
	return p;
}


malloc_t safe_realloc(p, len)
     malloc_t p;
     unsigned len;
{
        malloc_t po=p;
	if ((p = (p == NULL ? malloc(len) : realloc((malloc_t)p, len))) == NULL)
		(*safe_malloc_fail_handler)("safe_realloc", len);
	dprint(100,(debugfile,"safe_realloc(%X,%d)=%X\n",po,len,p));
	return p;
}

char *safe_strdup(s)
     CONST char *s;
{
	char *p;
	int Len = strlen(s)+1;
	if ((p = (char *) malloc(Len)) == NULL)
		(*safe_malloc_fail_handler)("safe_strdup", Len);
	dprint(100,(debugfile,"safe_strdup(%s)=%X\n",s,p));
	return strfcpy(p, s, Len);
}

#ifdef DEBUG
#undef free
void safe_free(x)
     malloc_t x;
{
    int i;
    char *p = x;
    dprint(100,(debugfile,"(safe)free(%X) ",x));


    if (x == NULL) {
      dprint(1,(debugfile,"** FREE OF NULL POINTER\n"));
#if 0
       fflush(debugfile);
       abort();
#endif
    }

    for (i = 0; i < 30 && p && p[i]; i++) {
      dprint(100,(debugfile,"%c",
		  isascii(p[i]) && isprint(p[i]) ? p[i] : '?'));
    }

    free(x);
    dprint(100,(debugfile,"\n"));
}
#endif
